/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

/**
 * 
 * Use in hibernate.cfg.xml <property
 * name="hibernate.current_session_context_class">thread</property>
 * 
 */
public class HibernateUtils {

	private static final SessionFactory sessionFactory;

	private static Logger log_ = Logger.getLogger(HibernateUtils.class);

	static {
		try {
			// Create the SessionFactory from hibernate.cfg.xml
			sessionFactory = new Configuration().configure()
					.buildSessionFactory();
		} catch (Throwable ex) {
			// Make sure you log the exception, as it might be swallowed
			System.err.println("Initial SessionFactory creation failed." + ex);
			throw new ExceptionInInitializerError(ex);
		}
	}

	public static SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	/**
	 * Get the current session and begin a transaction
	 * 
	 * @return
	 */
	public static Session getCurrentSession() {
		Session session = null;
		if (sessionFactory != null) {
			session = sessionFactory.getCurrentSession();
			session.beginTransaction();
		}
		return session;
	}

	public static Session openSession() {
		if (sessionFactory != null)
			sessionFactory.getCurrentSession();
		return null;
	}

	public static Object genericSave(Object object) throws Exception {
		try {
			Session session = getCurrentSession();
			session.save(object);
			session.getTransaction().commit();
		} catch (Exception e) {
			log_.info("Unable to save the object: " + object.getClass(), e);
			throw e;
		}
		return object;
	}
	
	public static Object genericUpdate(Object object) throws Exception {
		try {
			Session session = getCurrentSession();
			session.update(object);
			session.getTransaction().commit();
		} catch (Exception e) {
			log_.info("Unable to save the object: " + object.getClass(), e);
			throw e;
		}
		return object;
	}
	
	public static Object genericDelete(Object object) throws Exception {
		try {
			Session session = getCurrentSession();
			session.delete(object);
			session.getTransaction().commit();
		} catch (Exception e) {
			log_.info("Unable to save the object: " + object.getClass(), e);
			throw e;
		}
		return object;
	}

}
