/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.dao;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.Script;
import it.eng.spago4q.metadata.S4qEScript;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Order;
import org.hibernate.exception.ConstraintViolationException;

public class EScriptDAOHibImpl extends AbstractHibernateDAO implements IES4QDAO {
	
	private static Logger logger = Logger.getLogger(EScriptDAOHibImpl.class);

	static private String SCRIPT_NAME = "name";

	private String getScriptProperty(String property) {
		String toReturn = null;
		if (property != null && property.toUpperCase().equals("NAME"))
			toReturn = SCRIPT_NAME;
		return toReturn;
	}

	
	public boolean deleteObject(Integer id) throws EMFUserError {
		Session aSession = getCurrentSession();
		Transaction tx = null;
		try {
			tx = aSession.beginTransaction();
			S4qEScript s4qEScript = (S4qEScript) aSession.load(
					S4qEScript.class, id);
			aSession.delete(s4qEScript);
			tx.commit();

		} catch (ConstraintViolationException cve) {
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
			logger.error("Impossible to delete a Script ", cve);
			throw new EMFUserError(EMFErrorSeverity.WARNING, 10024);
		} catch (HibernateException e) {
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
			logger.error("Error while delete a Script ", e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 10012);
		} finally {
			aSession.close();
		}
		return true;

	}

	
	public Integer insertObject(Object object) throws EMFUserError {
		logger.debug("IN");
		Integer idToReturn;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();

			Script script = (Script) object;
			
			S4qEScript s4qEScript = new S4qEScript();

			saveEScriptValueToS4qEScript(script, s4qEScript);

			idToReturn = (Integer) aSession.save(s4qEScript);

			tx.commit();

		} catch (HibernateException he) {
			logException(he);
			if (tx != null)
				tx.rollback();
			throw new EMFUserError(EMFErrorSeverity.ERROR, 10030);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
			}
		}
		logger.debug("OUT");
		return idToReturn;
	}

	private void saveEScriptValueToS4qEScript(Script ascript, S4qEScript s4qEScript) {
			String name = ascript.getName();
			String description = ascript.getDescription();
			String scriptType = ascript.getScriptType();
			String script = ascript.getScript();

			s4qEScript.setName(name);
			s4qEScript.setDescription(description);
			s4qEScript.setScriptType(scriptType);
			s4qEScript.setScript(script);
	}

	
	public Object loadObjectById(Integer id) throws EMFUserError {
		logger.debug("IN");
		Script toReturn = null;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			S4qEScript hibScript = (S4qEScript) aSession.load(
					S4qEScript.class, id);
			toReturn = toScript(hibScript);
		} catch (HibernateException he) {
			logger.error("Error while loading the S4qEScript with id "
					+ ((id == null) ? "" : id.toString()), he);

			if (tx != null)
				tx.rollback();

			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
				logger.debug("OUT");
			}
		}
		return toReturn;


	}

	
	public List loadObjectList(String fieldOrder, String typeOrder)
			throws EMFUserError {
		logger.debug("IN");
		List<Script> toReturn = null;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			toReturn = new ArrayList<Script>();
			List<S4qEScript> toTransform = null;

			Criteria crit = aSession.createCriteria(S4qEScript.class);

			if (fieldOrder != null && typeOrder != null) {
				if (typeOrder.toUpperCase().trim().equals("ASC"))
					crit.addOrder(Order.asc(getScriptProperty(fieldOrder)));
				if (typeOrder.toUpperCase().trim().equals("DESC"))
					crit.addOrder(Order.desc(getScriptProperty(fieldOrder)));
			}
			toTransform = crit.list();

			for (Iterator<S4qEScript> iterator = toTransform.iterator(); iterator
					.hasNext();) {
				S4qEScript s4qEScript = iterator.next();

				Script aScript = toScript(s4qEScript);

				toReturn.add(aScript);
			}

		} catch (HibernateException he) {
			logger.error("Error while loading the list of EScript", he);

			if (tx != null)
				tx.rollback();
			throw new EMFUserError(EMFErrorSeverity.ERROR, 10031);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
				logger.debug("OUT");
			}
		}
		return toReturn;
	}

	protected static Script toScript(S4qEScript s4qEScript) {
		Script toReturn = null;
		if (s4qEScript != null) {
			toReturn = new Script();
			toReturn.setId(s4qEScript.getIdScript());
			toReturn.setName(s4qEScript.getName());
			toReturn.setDescription(s4qEScript.getDescription());
			toReturn.setScriptType(s4qEScript.getScriptType());
			toReturn.setScript(s4qEScript.getScript());
		}
		return toReturn;
	}

	
	public void modifyObject(Object object) throws EMFUserError {
		logger.debug("IN");
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			Script script = (Script)object;
			

			S4qEScript s4qEScript = (S4qEScript) aSession.load(
					S4qEScript.class, script.getId());

			saveEScriptValueToS4qEScript(script, s4qEScript);

			aSession.saveOrUpdate(s4qEScript);

			tx.commit();

		} catch (HibernateException he) {
			logException(he);

			if (tx != null)
				tx.rollback();

			throw new EMFUserError(EMFErrorSeverity.ERROR, 10032);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
			}
		}
		logger.debug("OUT");

	}

}
