/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.dao;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.ELog;
import it.eng.spago4q.metadata.S4qLog;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.exception.ConstraintViolationException;

public class LogDAOHibImpl extends AbstractHibernateDAO implements IES4QDAO {

	private static Logger logger = Logger.getLogger(LogDAOHibImpl.class);

	public boolean deleteObject(Integer id) throws EMFUserError {
		Session aSession = getCurrentSession();
		Transaction tx = null;
		try {
			tx = aSession.beginTransaction();
			S4qLog s4qLog = (S4qLog) aSession.load(S4qLog.class, id);
			aSession.delete(s4qLog);
			tx.commit();

		} catch (ConstraintViolationException cve) {
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
			logger.error("Impossible to delete a S4QLog ", cve);
			throw new EMFUserError(EMFErrorSeverity.WARNING, 10024);
		} catch (HibernateException e) {
			if (tx != null && tx.isActive()) {
				tx.rollback();
			}
			logger.error("Error while delete a Script ", e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} finally {
			aSession.close();
		}
		return true;

	}

	public Integer insertObject(Object object) throws EMFUserError {
		logger.debug("IN");
		Integer idToReturn;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();

			ELog log = (ELog) object;

			S4qLog s4qLog = new S4qLog();

			saveELogValueToS4qLog(log, s4qLog);

			idToReturn = (Integer) aSession.save(s4qLog);

			tx.commit();

		} catch (HibernateException he) {
			logException(he);
			if (tx != null)
				tx.rollback();
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
			}
		}
		logger.debug("OUT");
		return idToReturn;
	}

	private void saveELogValueToS4qLog(ELog log, S4qLog s4qlog) {
		Integer processId = log.getProcessId();
		Integer operationId = log.getOperationId();
		Integer extracted = log.getExtracted();
		Integer inserted = log.getInserted();
		Date operationTime = log.getOperationTime();

		s4qlog.setProcessId(processId);
		s4qlog.setOperationId(operationId);
		s4qlog.setExtracted(extracted);
		s4qlog.setInserted(inserted);
		s4qlog.setOperationTime(operationTime);
	}

	public Object loadObjectById(Integer id) throws EMFUserError {
		logger.debug("IN");
		ELog toReturn = null;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			S4qLog hibLog = (S4qLog) aSession.load(S4qLog.class, id);
			toReturn = toELog(hibLog);
		} catch (HibernateException he) {
			logger.error("Error while loading the S4qLog with id "
					+ ((id == null) ? "" : id.toString()), he);

			if (tx != null)
				tx.rollback();

			throw new EMFUserError(EMFErrorSeverity.ERROR, 10101);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
				logger.debug("OUT");
			}
		}
		return toReturn;
	}

	private ELog toELog(S4qLog hibLog) {
		ELog toReturn = null;
		if (hibLog != null) {
			toReturn = new ELog();
			toReturn.setIdLog(hibLog.getIdLog());
			toReturn.setProcessId(hibLog.getProcessId());
			toReturn.setOperationId(hibLog.getOperationId());
			toReturn.setExtracted(hibLog.getExtracted());
			toReturn.setInserted(hibLog.getInserted());
			toReturn.setOperationTime(hibLog.getOperationTime());
		}

		return toReturn;
	}

	public List loadObjectList(String fieldOrder, String typeOrder)
			throws EMFUserError {
		logger.debug("IN");
		List<ELog> toReturn = null;
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			toReturn = new ArrayList<ELog>();
			List<S4qLog> toTransform = null;

			Criteria crit = aSession.createCriteria(S4qLog.class);

			toTransform = crit.list();

			for (Iterator<S4qLog> iterator = toTransform.iterator(); iterator
					.hasNext();) {
				S4qLog s4qLog = iterator.next();

				ELog aELog = toELog(s4qLog);

				toReturn.add(aELog);
			}

		} catch (HibernateException he) {
			logger.error("Error while loading the list of S4Q ELog", he);

			if (tx != null)
				tx.rollback();
			// TODO inserire bundle e errore corretto
			throw new EMFUserError(EMFErrorSeverity.ERROR, 9104);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
				logger.debug("OUT");
			}
		}
		return toReturn;
	}

	public void modifyObject(Object object) throws EMFUserError {
		logger.debug("IN");
		Session aSession = null;
		Transaction tx = null;
		try {
			aSession = getCurrentSession();
			tx = aSession.beginTransaction();
			ELog eLog = (ELog) object;

			S4qLog s4qLog = (S4qLog) aSession.load(S4qLog.class, eLog
					.getIdLog());

			saveELogValueToS4qLog(eLog, s4qLog);

			aSession.saveOrUpdate(s4qLog);

			tx.commit();

		} catch (HibernateException he) {
			logException(he);

			if (tx != null)
				tx.rollback();

			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
			}
		}
		logger.debug("OUT");

	}

}
