/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.DataSourceParameter;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.bo.EOperationParameter;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;

import java.util.Iterator;
import java.util.List;

public abstract class AbstractExtractor implements ExtractorInterface {

	private EOperation eOperation = null;
	private List operationParameter = null;
	private List dataSourceParameter = null;
	private static final String EOPERATIONPARAMETER_DAO = "EOperationParameterDAO";
	private static final String EDATASOURCEPARAMETER_DAO = "EDataSourceParameterDAO";
	
	
	public List<GenericItemInterface> execute()  throws EMFUserError {
		List<GenericItemInterface> toReturn = null;
		setUp();
		try{
			toReturn = extract();
		}catch (EMFUserError emfUE){
			
			throw emfUE;			
		} finally{
			tearDown();
		}
		return toReturn;
	}

	public void init(EOperation operation) throws EMFUserError {
		this.eOperation = operation;
		
		IES4QFKDAO dataSourceParameterDAO = (IES4QFKDAO) DAOFactory.getDAO(EDATASOURCEPARAMETER_DAO);
		dataSourceParameter = dataSourceParameterDAO.loadObjectList(getEOperation().getDataSource().getId(), null, null);
		
		
		IES4QFKDAO operationParameterDAO = (IES4QFKDAO) DAOFactory.getDAO(EOPERATIONPARAMETER_DAO);
		operationParameter = operationParameterDAO.loadObjectList(getEOperation().getId(), null, null);	
	}
	
	protected EOperation getEOperation(){
		return eOperation;
	}
	
	public List getOperationParameter(){
		return operationParameter;
	}
	
	public List getDataSourceParameter() {
		return dataSourceParameter;
	}
	
	protected String readDataSourceParameterValue(String key) {
		String toReturn = null;
		Iterator it = (Iterator) getDataSourceParameter().iterator();
		while (it.hasNext()) {
			DataSourceParameter dataSourceParameter = (DataSourceParameter) it
					.next();
			if (dataSourceParameter.getName().equals(key))
				toReturn = dataSourceParameter.getValue();
		}
		return toReturn;
	}

	protected String readOperationParameterValue(String key) {
		String toReturn = null;
		Iterator it = (Iterator) getOperationParameter().iterator();
		while (it.hasNext()) {
			EOperationParameter operationParameter = (EOperationParameter) it
					.next();
			if (operationParameter.getName().equals(key))
				toReturn = operationParameter.getValue();
		}
		return toReturn;
	}
	
	abstract protected void setUp();

	abstract protected void tearDown();
	
	abstract protected List<GenericItemInterface> extract() throws EMFUserError;
	
	
}
