/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.mappers.SQLMapper;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.InterfaceField;
import it.eng.spago4q.dao.AbstractHibernateDAO;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.utilities.ExtractorConstants;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.hibernate.Session;

public class TableHandler extends AbstractHibernateDAO {

	private static Logger logger = Logger.getLogger(TableHandler.class);

	private static final String TABLE_NAME_PLACE_HOLDER = "@tablename";
	private static final String COLUMN_NAME_TYPE_PLACE_HOLDER = "@columnNameType";
	private static final String COLUMN_NAME_PLACE_HOLDER = "@columns";
	private static final String VALUES_PLACE_HOLDER = "@values";
	private static final String WHERE_PLACE_HOLDER = "@where";
	private static final String SELECT_PLACE_HOLDER = "@select";

	protected static String createDropTable(String tableName) {
		String commandString = SQLStatements.getStatement("DROP_TABLE");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		return commandString;
	}

	// protected static String createAlterTableAdd(String tableName,
	// List interfaceFieldList) {
	// String prefix = "ALTER TABLE " + tableName + "ADD";
	//
	// return prefix;
	//
	// // ALTER TABLE table_name
	// // ADD ( column_1 column-definition,
	// // column_2 column-definition,
	// // ...
	// // column_n column_definition );
	// //
	// }
	//
	// protected static String createAlterTableDrop(String tableName,
	// List interfaceFieldList) {
	// String prefix = "ALTER TABLE " + tableName + "DROP COLUMN";
	//
	// return prefix;
	//
	// }

	/**
	 * Creates a ago DataConnection object starting from a sql connection.
	 * 
	 * @param con
	 *            Connection to the export database
	 * 
	 * @return The Spago DataConnection Object
	 * 
	 * @throws EMFInternalError
	 *             the EMF internal error
	 */
	public static DataConnection getDataConnection(Connection con)
			throws EMFInternalError {
		DataConnection dataCon = null;
		try {
			Class mapperClass = Class
					.forName("it.eng.spago.dbaccess.sql.mappers.OracleSQLMapper");
			SQLMapper sqlMapper = (SQLMapper) mapperClass.newInstance();
			dataCon = new DataConnection(con, "2.1", sqlMapper);
		} catch (Exception e) {
			throw new EMFInternalError(EMFErrorSeverity.ERROR,
					"cannot build spago DataConnection object");
		}
		return dataCon;
	}

	protected String createQuery(String tableName, List interfaceFieldList)
			throws EMFUserError {
		String columnNameType = "";
		for (Iterator iterator = interfaceFieldList.iterator(); iterator
				.hasNext();) {
			InterfaceField interfaceField = (InterfaceField) iterator.next();

			if (!isValidColumName(interfaceField.getName())) {
				throw new EMFUserError(EMFErrorSeverity.ERROR, 10035);
			}

			columnNameType += ", " + interfaceField.getName() + " ";
			IES4QFKDAO eDomainValueDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EDomainValueDAO");
			List domainValueList = eDomainValueDAO.loadObjectList(
					interfaceField.getId(), null, null);

			if (domainValueList == null
					|| (domainValueList != null && domainValueList.isEmpty())) {
				columnNameType += typeConverter(interfaceField.getFieldType());
			} else {
				columnNameType += typeConverter(ExtractorConstants.INTEGER);
			}
		}

		String commandString = SQLStatements.getStatement("CREATE_TABLE");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		commandString = commandString.replace(COLUMN_NAME_TYPE_PLACE_HOLDER,
				columnNameType);

		logger.debug("Return this Create query:" + commandString);
		System.out.println("Return this Create query:" + commandString);
		return commandString;

	}

	protected static String createInsert(String tableName,
			InterfaceItem interfaceItem, Integer logId) {
		String columns = "";
		String values = logId + ", ";
		Set<String> keySet = interfaceItem.getHashMap().keySet();

		for (Iterator<String> iterator = keySet.iterator(); iterator.hasNext();) {
			String key = (String) iterator.next();

			columns += key;
			
			values += getDBValue(interfaceItem, key);

			if (iterator.hasNext()) {
				columns += ",";
				values += ",";
			}
		}

		String commandString = SQLStatements.getStatement("INSERT_EXTRACTION");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		commandString = commandString
				.replace(COLUMN_NAME_PLACE_HOLDER, columns);
		commandString = commandString.replace(VALUES_PLACE_HOLDER, values);

		logger.debug("Return this Insert query:" + commandString);
		System.out.println("Return this Insert query:" + commandString);
		return commandString;
	}

	/**
	 * Create the UPDATE query that sets end_time to current time.
	 * 
	 * @param tableName
	 *            the name of the table to update.
	 * @param interfaceItem
	 *            the list of interface field where to find the key value.
	 * @return the UPDATE query.
	 */
	protected static String createUpdate(String tableName,
			InterfaceItem interfaceItem) {
		String where = "";

		Set<String> keySet = interfaceItem.getHashMap().keySet();

		for (Iterator<String> iterator = keySet.iterator(); iterator.hasNext();) {
			String key = (String) iterator.next();
			if (interfaceItem.getValue(key).isKey()) {
				where += " AND " + key + " = " + getDBValue(interfaceItem, key);
			}
		}

		String commandString = SQLStatements.getStatement("UPDATE_EXTRACTION");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		commandString = commandString.replace(WHERE_PLACE_HOLDER, where);

		logger.debug("Return this Update query:" + commandString);
		System.out.println("Return this Update query:" + commandString);
		return commandString;
	}

	protected static String createSelect(String tableName,
			InterfaceItem interfaceItem) {
		String select = "";
		String where = "";

		Set<String> keySet = interfaceItem.getHashMap().keySet();

		for (Iterator<String> iterator = keySet.iterator(); iterator.hasNext();) {
			String key = (String) iterator.next();
			if (interfaceItem.getValue(key).isKey()){
				// add key fields in where conditions
				String value = getDBValue(interfaceItem, key);
				if (value.equals("null")){
					where += " AND " + key + " is "
					+ value + "";
					
				}else {
					where += " AND " + key + " = "
					+ value + "";
					
				}
				
			}

			if (interfaceItem.getValue(key).isSensible()){
				select += ", " + key;
				// add sensible fields in where conditions
				String value = getDBValue(interfaceItem, key);
				if (value.equals("null")){
					where += " AND " + key + " is "
					+ value + "";
					
				}else {
					where += " AND " + key + " = "
					+ value + "";
					
				}
				
			}
		}

		String commandString = SQLStatements.getStatement("SELECT_EXTRACTION");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		commandString = commandString.replace(SELECT_PLACE_HOLDER, select);
		commandString = commandString.replace(WHERE_PLACE_HOLDER, where);

		System.out.println("Return this Select query:" + commandString);
		logger.debug("Return this Select query:" + commandString);
		return commandString;
	}

	protected SourceBean selectQueryExecutor(String query) throws EMFUserError {
		Session aSession = null;
		SourceBean result = null;
		DataConnection dataConnection = null;
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		try {
			aSession = getCurrentSession();
			Connection jdbcConnection = aSession.connection();
			dataConnection = getDataConnection(jdbcConnection);
			sqlCommand = dataConnection.createSelectCommand(query);
			dataResult = sqlCommand.execute();
			ScrollableDataResult scrollableDataResult = (ScrollableDataResult) dataResult
					.getDataObject();
			result = scrollableDataResult.getSourceBean();
		} catch (EMFInternalError e) {
			throw new EMFUserError(e.getCategory(), e.getDescription());
		} finally {
			Utils.releaseResources(dataConnection, sqlCommand, dataResult);
		}
		return result;
	}

	protected void queryExecutor(String query) throws EMFUserError {
		Session aSession = null;
		DataConnection dataConnection = null;
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		try {
			aSession = getCurrentSession();
			Connection jdbcConnection = aSession.connection();
			dataConnection = getDataConnection(jdbcConnection);
			sqlCommand = dataConnection.createInsertCommand(query);
			sqlCommand.execute();
		} catch (EMFInternalError e) {
			throw new EMFUserError(e.getCategory(), e.getDescription());
		} finally {
			Utils.releaseResources(dataConnection, sqlCommand, dataResult);
		}
	}

	protected boolean hasThisTable(String tableName) throws EMFUserError {
		boolean toReturn = false;
		Session aSession = null;
		DatabaseMetaData dbmd = null;
		Connection connection = null;
		try {
			aSession = getCurrentSession();
			connection = aSession.connection();
			dbmd = connection.getMetaData();

			// Specify the type of object; in this case we want tables
			String[] types = { "TABLE" };
			ResultSet resultSet = dbmd.getTables(null, null, "%", types);

			// Get the table names
			while (resultSet.next()) {
				// Get the table name
				String dbTableName = resultSet.getString(3);
				if (dbTableName.toUpperCase().equals(tableName)) {
					toReturn = true;
					break;
				}
			}
		} catch (SQLException e) {
			// abstractHibernateDao.logException(e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 100);

		} finally {
			if (aSession != null) {
				if (aSession.isOpen())
					aSession.close();
			}
		}
		return toReturn;
	}

	private static String typeConverter(String type) {
		String toReturn = "";
		ConfigSingleton configSingleton = ConfigSingleton.getInstance();
		SourceBean daoConfigSourceBean = (SourceBean) configSingleton
				.getFilteredSourceBeanAttribute("SPAGO4Q.DB-CONF.DB-TYPE",
						"name", type);
		toReturn = (String) daoConfigSourceBean.getAttribute("conversion");

		return toReturn;
	}

	protected static String transformTableName(String tableName) {
		return "FT_" + tableName.trim().replace(" ", "").toUpperCase();
	}

	private static boolean isValidColumName(String columName) {
		boolean toReturn = true;
		if (columName != null && columName.toUpperCase().equals("ID"))
			toReturn = false;
		if (columName != null && columName.toUpperCase().equals("BEGIN_TIME"))
			toReturn = false;
		if (columName != null && columName.toUpperCase().equals("END_TIME"))
			toReturn = false;
		if (columName != null && columName.toUpperCase().equals("ID_LOG"))
			toReturn = false;
		return toReturn;
	}

	private static String getDBValue(InterfaceItem interfaceItem, String key) {
		String toReturn = "";
		if (interfaceItem.getValue(key).getValue() != null){
			toReturn = "'" + interfaceItem.getValue(key).getValue().toString() + "'";
		} else {
			toReturn = "null";
		}
		return toReturn;
	}

}
