/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors.script;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.Script;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QDAO;

import java.util.HashMap;

import javax.script.ScriptException;

import org.apache.log4j.Logger;

public class ScriptCache {
	
	private static transient Logger logger = Logger.getLogger(ScriptCache.class);

	private HashMap<Integer, IMappingScript> scripts;

	public ScriptCache() {
		scripts = new HashMap<Integer, IMappingScript>();
	}

	public IMappingScript getScript(Integer scriptId) {
		IMappingScript toReturn = scripts.get(scriptId);
		if (toReturn == null) {
			toReturn = getScriptFromDB(scriptId);
			addScript(scriptId, toReturn);
		}
		return toReturn;
	}

	private void addScript(Integer scriptId, IMappingScript script) {
		scripts.put(scriptId, script);
	}

	private IMappingScript getScriptFromDB(Integer scriptId) {
		IMappingScript toReturn = null;

		IES4QDAO scriptDAO;
		try {
			scriptDAO = (IES4QDAO) DAOFactory.getDAO("EScriptDAO");
			Script script = (Script) scriptDAO.loadObjectById(scriptId);
			if (script != null && script.getScript() != null) {
				toReturn = ScriptCreator
						.createScriptObject(script.getScript());
			}
		} catch (ClassCastException e) {
			logger.error("Problem douring creation of script with id" + scriptId , e);
		} catch (ScriptException e) {
			logger.error("Problem douring creation of script with id" + scriptId , e);
		} catch (EMFUserError e1) {
			logger.error("Problem douring load of Script Object with id" + scriptId , e1);
		}
		return toReturn;
	}
}
