package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.bo.InterfaceType;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QDAO;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.testUtils.AbstractDBTestCase;

import java.util.Iterator;
import java.util.List;

public class TableHandlerTest extends AbstractDBTestCase {
	private static final String FLAT_FILE = "test/s4qFlatFile.xml";

	public TableHandlerTest(String name) {
		super(name);
	}

	public void testCreateQuery() {
		try {
			IES4QDAO interfaceTypeDAO = (IES4QDAO) DAOFactory
					.getDAO("EInterfaceTypeDAO");

			InterfaceType eInterfaceType = (InterfaceType) interfaceTypeDAO
					.loadObjectById(1);

			IES4QFKDAO interfaceFieldDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EInterfaceFieldDAO");
			List interfaceFieldList = interfaceFieldDAO.loadObjectList(
					eInterfaceType.getId(), null, null);

			assertEquals(3, interfaceFieldList.size());

			String query = new TableHandler().createQuery(eInterfaceType
					.getName(), interfaceFieldList);
			assertFalse(query.isEmpty());

		} catch (EMFUserError e) {
			fail();
		}
	}

	public void testExecuteQuery() {
		TableHandler tableCreator = new TableHandler();

		String tableName = null;
		try {
			IES4QDAO interfaceTypeDAO = (IES4QDAO) DAOFactory
					.getDAO("EInterfaceTypeDAO");

			InterfaceType eInterfaceType = (InterfaceType) interfaceTypeDAO
					.loadObjectById(1);

			tableName = TableHandler.transformTableName(eInterfaceType
					.getTableName());

			IES4QFKDAO interfaceFieldDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EInterfaceFieldDAO");
			List interfaceFieldList = interfaceFieldDAO.loadObjectList(
					eInterfaceType.getId(), null, null);

			assertEquals(3, interfaceFieldList.size());
			String query = new TableHandler().createQuery(tableName,
					interfaceFieldList);
			System.out.println(query);
			assertFalse(query.isEmpty());

			tableCreator.queryExecutor(query);
		} catch (Exception e) {
			fail();
		}

		try {
			assertTrue(tableCreator.hasThisTable(tableName));
		} catch (Exception e) {
			fail();
		}

	}

	public void testCreateInsert() {
		try {
			OperationCoordinatorInterface oCI = OperationCoordinatorFactory
					.create(2);
			assertEquals(DefaultCoordinator.class, oCI.getClass());

			assertEquals(1, oCI.getOperationsList().size());

			assertEquals("it.eng.spago4q.extractors.db.DbExtractor",
					((EOperation) oCI.getOperationsList().get(0))
							.getDataSource().getSourceType()
							.getExtractorClass());

			AbstractExtractor extractor = (AbstractExtractor) Class.forName(
					((EOperation) oCI.getOperationsList().get(0))
							.getDataSource().getSourceType()
							.getExtractorClass()).newInstance();

			extractor.init((EOperation) oCI.getOperationsList().get(0));

			assertEquals(4, extractor.getDataSourceParameter().size());
			assertEquals(1, extractor.getOperationParameter().size());

			List<GenericItemInterface> genericItemList = extractor.execute();

			assertEquals(2, genericItemList.size());

//			List<InterfaceItem> interfaceItem = MapMaker.mapping(
//					(EOperation) oCI.getOperationsList().get(0),
//					genericItemList);

//			assertEquals(2, interfaceItem.size());
//
//			String tableNameToTransform = ((EOperation) oCI.getOperationsList()
//					.get(0)).getInterfaceType().getTableName();
//
//			String tableName = TableHandler
//					.transformTableName(tableNameToTransform);
//
//			for (Iterator iterator = interfaceItem.iterator(); iterator
//					.hasNext();) {
//				InterfaceItem interfaceItem2 = (InterfaceItem) iterator.next();
//
//				System.out.println(TableHandler.createInsert(tableName,
//						interfaceItem2,0));
//			}

		} catch (Exception e) {
			fail();
		}
	}
	
	public void testAllExtractionProcess() {
		try {
			OperationCoordinatorInterface oCI = OperationCoordinatorFactory
					.create(2);
			assertEquals(DefaultCoordinator.class, oCI.getClass());

			assertEquals(1, oCI.getOperationsList().size());

			assertEquals("it.eng.spago4q.extractors.db.DbExtractor",
					((EOperation) oCI.getOperationsList().get(0))
							.getDataSource().getSourceType()
							.getExtractorClass());

			AbstractExtractor extractor = (AbstractExtractor) Class.forName(
					((EOperation) oCI.getOperationsList().get(0))
							.getDataSource().getSourceType()
							.getExtractorClass()).newInstance();

			extractor.init((EOperation) oCI.getOperationsList().get(0));

			assertEquals(4, extractor.getDataSourceParameter().size());
			assertEquals(1, extractor.getOperationParameter().size());

			List<GenericItemInterface> genericItemList = extractor.execute();

			assertEquals(2, genericItemList.size());

//			List<InterfaceItem> interfaceItem = MapMaker.mapping(
//					(EOperation) oCI.getOperationsList().get(0),
//					genericItemList);
//
//			assertEquals(2, interfaceItem.size());
//
//			// create table
//
//			new FactTableManager().saveFactTable(((EOperation) oCI
//					.getOperationsList().get(0)).getInterfaceType().getId());
//			
//			String tableNameToTransform = ((EOperation) oCI.getOperationsList()
//					.get(0)).getInterfaceType().getTableName();
//
//			String tableName = TableHandler
//					.transformTableName(tableNameToTransform);
//			
//			
//			
//			for (Iterator iterator = interfaceItem.iterator(); iterator
//					.hasNext();) {
//				InterfaceItem interfaceItem2 = (InterfaceItem) iterator.next();
//				
//				new FactTableManager().loadData(1, interfaceItem2, 0);
//				
//				
//			}
//			
//			
//			
//			new FactTableManager().dropFactTable(((EOperation) oCI
//					.getOperationsList().get(0)).getInterfaceType().getId());
//
//
		} catch (Exception e) {
			fail();
		}
	}

	

	@Override
	protected String getFlatFile() {
		return FLAT_FILE;
	}

}
